@@ -39,14 +39,14 @@ typedef struct _GtkSourceLanguageManagerPrivate GtkSourceLanguageManagerPrivate;
 
 struct _GtkSourceLanguageManager
 {
-	GObject parent;
+	GObject parent_instance;
 
 	GtkSourceLanguageManagerPrivate *priv;
 };
 
 struct _GtkSourceLanguageManagerClass
 {
-	GObjectClass              parent_class;
+	GObjectClass parent_class;
 
 	/* Padding for future expansion */
 	void (*_gtk_source_reserved1) (void);
@@ -59,17 +59,23 @@ struct _GtkSourceLanguageManagerClass
 GType			  gtk_source_language_manager_get_type			(void) G_GNUC_CONST;
 
 GtkSourceLanguageManager *gtk_source_language_manager_new			(void);
+
 GtkSourceLanguageManager *gtk_source_language_manager_get_default		(void);
 
-const GSList		 *gtk_source_language_manager_get_available_languages	(GtkSourceLanguageManager *lm);
+G_CONST_RETURN gchar* G_CONST_RETURN *
+			  gtk_source_language_manager_get_search_path		(GtkSourceLanguageManager *lm);
 
-GtkSourceLanguage	 *gtk_source_language_manager_get_language_by_id	(GtkSourceLanguageManager *lm,
-										 const gchar              *id);
-
-gchar			**gtk_source_language_manager_get_search_path		(GtkSourceLanguageManager *lm);
 void			  gtk_source_language_manager_set_search_path		(GtkSourceLanguageManager *lm,
 										 gchar                   **dirs);
 
+G_CONST_RETURN gchar* G_CONST_RETURN *
+			  gtk_source_language_manager_get_language_ids		(GtkSourceLanguageManager *lm);
+
+GtkSourceLanguage	 *gtk_source_language_manager_get_language		(GtkSourceLanguageManager *lm,
+										 const gchar              *id);
+
+
+
 G_END_DECLS
 
 #endif /* __GTK_SOURCE_LANGUAGE_MANAGER_H__ */
